#ifndef VDOP_FILE
#define VDOP_FILE


/** @brief  Element-wise vector divison x_k = y_k/z_k.
 *  A stride @p iy =0 for the source stride adds a constant value stored in @p y[0].
 *  The same holds for vector @p z.
 *
 * @param[in]  n  number of elements to divide
 * @param[out] x  target vector
 * @param[in]  ix stride for target vector @p x
 * @param[in]  y  source vector
 * @param[in]  iy stride for source vector @p y
 * @param[in]  z  source vector
 * @param[in]  iz stride for source vector @p z
 *
 */
void vddiv(int n,  double x[], int ix,
           double const y[], int iy,
           double const z[], int iz);

/** @brief  Element-wise daxpy operation x(k) = y(k) + alpha*z(k).
 *
 * @param[in]  n  number of vector elements in each vector
 * @param[out] x  target vector
 * @param[in]  y  source vector
 * @param[in]  alpha  scalar
 * @param[in]  z  source vector
 *
 */
void vdaxpy(int n, double x[],   double const y[],
            double alpha, double const z[] );


/** @brief  Calculates the Euclidian inner product of two vectors.
 *
 * @param[in]  n  number of elements in both vectors.
 * @param[in]  x source vector
 * @param[in]  y source vector
 * @return Euclidian inner product @f$\langle x,y \rangle@f$
 *
 */
double dscapr(int n, double const x[], double const y[]);



#endif
